require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.os.Vibrator"  -- Import Vibrator

-- Function to destroy the recognizer after usage
function destroyRecognizer(recognizer)
  if recognizer then
    recognizer.destroy()
    recognizer = nil
  end
  return true
end

-- Function to start speech recognition and handle results
function startSpeechRecognition()
  local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

  -- Create the recognition listener
  local recognitionListener = RecognitionListener {
    onResults = function(results)
      local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
      if resultArray ~= nil and resultArray.size() > 0 then
        local recognizedText = resultArray.get(0) -- Original recognized text

        -- Perform word replacement and text transformations
        local replacedText = replaceWords(recognizedText)

        -- Add the Nepali period "।" at the end of the text
        replacedText = replacedText .. "।"

        -- Insert the transformed text into the EditText and speak the result
        local editText = service.getEditText()
        service.insertText(editText, replacedText .. " ") -- Insert the text obtained after replacement
        service.speak(replacedText)

        -- Add vibration functionality
        local vibrator = this.getSystemService(Context.VIBRATOR_SERVICE) -- Get Vibrator service
        if vibrator ~= nil then
          vibrator.vibrate(500) -- Vibrate for 500 milliseconds
        end
      end
      destroyRecognizer(speechRecognizer) -- Destroy recognizer after use
    end,
    onError = function()
      destroyRecognizer(speechRecognizer) -- Handle error and destroy recognizer
      return false
    end
  }

  -- Set up the intent for speech recognition
  local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ne-NP") -- Set recognition language to Nepali
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

  -- Start the speech recognizer and set the listener
  speechRecognizer.startListening(recognizerIntent)
  speechRecognizer.setRecognitionListener(recognitionListener)

  return true
end

-- Function to replace words with appropriate symbols
function replaceWords(text)
  local wordReplacements = {
    ["comma"] = ",",
    ["period"] = "।",
    ["question mark"] = "?",
    ["exclamation mark"] = "!",
    ["ending symbol"] = "।" -- Add the "ending symbol" applicable in Nepali
  }

  -- Replace words in the text with corresponding symbols
  for originalWord, replacementWord in pairs(wordReplacements) do
    text = text:gsub(originalWord, replacementWord)
  end

  return text
end

-- Start speech recognition
startSpeechRecognition()